package com.sun.jndi.dns;

import javax.naming.CommunicationException;
import javax.naming.NameNotFoundException;
import javax.naming.NamingException;

class Resolver
{
    private DnsClient dnsClient;
    private int timeout;
    private int retries;
    
    Resolver(String[] array, int timeout, int retries) throws NamingException {
        this.timeout = timeout;
        this.retries = retries;
        this.dnsClient = new DnsClient(array, timeout, retries);
    }
    
    public void close() {
        this.dnsClient.close();
        this.dnsClient = null;
    }
    
    ResourceRecords query(DnsName dnsName, int n, int n2, boolean b, boolean b2) throws NamingException {
        return this.dnsClient.query(dnsName, n, n2, b, b2);
    }
    
    ResourceRecords queryZone(DnsName dnsName, int n, boolean b) throws NamingException {
        DnsClient dnsClient = new DnsClient(this.findNameServers(dnsName, b), this.timeout, this.retries);
        try {
            return dnsClient.queryZone(dnsName, n, b);
        }
        finally {
            dnsClient.close();
        }
    }
    
    DnsName findZoneName(DnsName dnsName, int n, boolean b) throws NamingException {
        dnsName = (DnsName)dnsName.clone();
        while (dnsName.size() > 1) {
            ResourceRecords query = null;
            try {
                query = this.query(dnsName, n, 6, b, false);
            }
            catch (NameNotFoundException ex) {
                throw ex;
            }
            catch (NamingException ex2) {}
            if (query != null) {
                if (query.answer.size() > 0) {
                    return dnsName;
                }
                for (int i = 0; i < query.authority.size(); ++i) {
                    ResourceRecord resourceRecord = (ResourceRecord)query.authority.elementAt(i);
                    if (resourceRecord.getType() == 6) {
                        DnsName name = resourceRecord.getName();
                        if (dnsName.endsWith(name)) {
                            return name;
                        }
                    }
                }
            }
            dnsName.remove(dnsName.size() - 1);
        }
        return dnsName;
    }
    
    ResourceRecord findSoa(DnsName dnsName, int n, boolean b) throws NamingException {
        ResourceRecords query = this.query(dnsName, n, 6, b, false);
        int i = 0;
        while (i < query.answer.size()) {
            ResourceRecord resourceRecord = (ResourceRecord)query.answer.elementAt(i);
            if (resourceRecord.getType() == 6) {
                return resourceRecord;
            }
            ++i;
        }
        return null;
    }
    
    private String[] findNameServers(DnsName dnsName, boolean b) throws NamingException {
        ResourceRecords query = this.query(dnsName, 1, 2, b, false);
        String[] array = new String[query.answer.size()];
        int i = 0;
        while (i < array.length) {
            ResourceRecord resourceRecord = (ResourceRecord)query.answer.elementAt(i);
            if (resourceRecord.getType() != 2) {
                throw new CommunicationException("Corrupted DNS message");
            }
            array[i] = (String)resourceRecord.getRdata();
            array[i] = array[i].substring(0, array[i].length() - 1);
            ++i;
        }
        return array;
    }
}
